@@ -47,6 +47,10 @@ static inline int skb_data_offset(struct sk_buff *skb, size_t *data_offset, size
 
 	udp = udp_hdr(skb);
 	*data_offset = (u8 *)udp - skb->data;
+	if (unlikely(*data_offset > U16_MAX)) {
+		net_dbg_ratelimited("Packet has offset at impossible location from %pISpfsc\n", &addr);
+		return -EINVAL;
+	}
 	if (unlikely(*data_offset + sizeof(struct udphdr) > skb->len)) {
 		net_dbg_ratelimited("Packet isn't big enough to have UDP fields from %pISpfsc\n", &addr);
 		return -EINVAL;
@@ -269,7 +273,7 @@ void packet_receive(struct wireguard_device *wg, struct sk_buff *skb)
 	static const u8 addr;
 #endif
 
-	if (skb_data_offset(skb, &offset, &len) < 0)
+	if (unlikely(skb_data_offset(skb, &offset, &len) < 0 || !len))
 		goto err;
 	switch (message_determine_type(skb->data + offset, len)) {
 	case MESSAGE_HANDSHAKE_INITIATION:
