@@ -6,30 +6,29 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <syscall.h>
-#include <unistd.h>
 #include <string.h>
+#include <fcntl.h>
 
 #include "curve25519.h"
 #include "base64.h"
+#include "subcommands.h"
 
-#ifdef __NR_getrandom
-static inline ssize_t get_random_bytes(uint8_t *out, size_t len)
-{
-	return syscall(__NR_getrandom, out, len, 0);
-}
-#else
-#include <fcntl.h>
 static inline ssize_t get_random_bytes(uint8_t *out, size_t len)
 {
 	ssize_t ret;
-	int fd = open("/dev/urandom", O_RDONLY);
+	int fd;
+#ifdef __NR_getrandom
+	ret = syscall(__NR_getrandom, out, len, 0);
+	if (ret >= 0)
+		return ret;
+#endif
+	fd = open("/dev/urandom", O_RDONLY);
 	if (fd < 0)
 		return fd;
 	ret = read(fd, out, len);
 	close(fd);
 	return ret;
 }
-#endif
 
 int genkey_main(int argc, char *argv[])
 {
@@ -37,6 +36,11 @@ int genkey_main(int argc, char *argv[])
 	char private_key_base64[b64_len(CURVE25519_POINT_SIZE)];
 	struct stat stat;
 
+	if (argc != 1) {
+		fprintf(stderr, "Usage: %s %s\n", PROG_NAME, argv[0]);
+		return 1;
+	}
+
 	if (!fstat(STDOUT_FILENO, &stat) && S_ISREG(stat.st_mode) && stat.st_mode & S_IRWXO)
 		fputs("Warning: writing to world accessible file.\nConsider setting the umask to 077 and trying again.\n", stderr);
 
@@ -47,9 +51,8 @@ int genkey_main(int argc, char *argv[])
 	if (argc && !strcmp(argv[0], "genkey"))
 		curve25519_normalize_secret(private_key);
 
-	if (b64_ntop(private_key, sizeof(private_key), private_key_base64, sizeof(private_key_base64)) < 0) {
-		errno = EINVAL;
-		perror("b64");
+	if (b64_ntop(private_key, sizeof(private_key), private_key_base64, sizeof(private_key_base64)) != sizeof(private_key_base64) - 1) {
+		fprintf(stderr, "%s: Could not convert key to base64\n", PROG_NAME);
 		return 1;
 	}
 
